@@ -7,6 +7,7 @@ import (
 	igntypes "github.com/coreos/ignition/config/v2_2/types"
 	validate "github.com/coreos/ignition/config/validate"
 	"github.com/golang/glog"
+	mcfgv1 "github.com/openshift/machine-config-operator/pkg/apis/machineconfiguration.openshift.io/v1"
 	errors "github.com/pkg/errors"
 )
 
@@ -41,3 +42,14 @@ func ValidateIgnition(cfg igntypes.Config) error {
 	}
 	return nil
 }
+
+// ValidateMachineConfig validates that given MachineConfig Spec is valid.
+func ValidateMachineConfig(cfg mcfgv1.MachineConfigSpec) error {
+	if !(cfg.KernelType == "" || cfg.KernelType == "default" || cfg.KernelType == "realtime") {
+		return errors.Errorf("kernelType=%s is invalid", cfg.KernelType)
+	}
+	if err := ValidateIgnition(cfg.Config); err != nil {
+		return err
+	}
+	return nil
+}
